% Dominant-subordinate hierarchy contest

% INPUT
x = 0.6;    % reproductive output of solitary queen in new colony
k = 2;      % combined reproductive output of a pair if together
r = 0.5;    % relatedness of the two queens
f = 0.2;    % probability that subordinate will win the fight

% OUTPUT will be an ESS value of p
p  = linspace(0,1,1001); % fraction of the resources given to subordinate

%% Calculate direct and inclusive fitness
% the fitness will be stored as a matrix with 3 rows and as many columns as
% vector p has. Each entry (i,j) of the matrix will correspond to a fitness
% when dominant gives p(j) and subordinate stays (i=1), leaves (i=2), fight
% (i=3)
one = ones(1,length(p));    % vector of 1's to keep dimensions same

S_direct = [k*p;            % subordinate stays with the dominant queen
            x*one;          % subordinate leaves and sets up new colony
            f*one];         % subordinate fights with the dominant queen
D_direct = [k*(1-p);        % subordinate stays
            one;            % subordinate leaves
            (1-f)*one];     % subordinate fights

S_inc = S_direct + r*D_direct;  % subordinate's inclusive fitness 
D_inc = D_direct + r*S_direct;  % dominant's inclusive fitness

%% Actual ESS analysis
% done by backward induction (dominant chooses p and subordinate reacts
% so first get best response of subordinate to any p and then let
% dominant choose the best p
max_S = max(S_inc,[],1);           % maximal gain for subordinate
BR_S = max_S(ones(1,3), :)==S_inc; % strategy for maximal gain
% find corresponding reward to dominant and which p it should play
D_Reward = D_inc.*BR_S;      % gain to dominant if subordinant plays BR
ESS=(max(D_Reward(BR_S))==D_Reward); % 1 at (action, p) if the pair is ESS

%% Display ESS
% one goes through all elements of the ESS and displays all that are 1
% since sometimes the action of the subordinate does not depend on p
% these cases are handled separately to avoid displaying too many results
Actions={'stays', 'leaves', 'fights'}; % actions of subordinate
for action = 1:3
    pESS = p(ESS(action,:)==1);   % ESS values of p for this action
    numESS = length(pESS);        % how many ESS values of p there are
    if (numESS==1)                % one p yields ESS, others do not
        disp(['ESS: Dominant gives p=', num2str(pESS), ...
              ',  Subordinate ', Actions{action}]);
    elseif (numESS>1)             % more than one p yields an ESS
        % in this case it will be a range of p's in an interval
        disp(['ESS: Dominant gives any p between, ', ...
                num2str(min(pESS)), ' and ', num2str(max(pESS)), ...
              ' Subordinate ', Actions{action},' regardless of p.']);
    end
 end % for action
