function IPD_tournament
% implementation of a round robin tournament of iterated PD game
% User specifies the PD payoff matrix, probability of next round 
% and probability of an error (in execution) of the move
% User also specifies the list of strategies and their definition

%% User defined parameters
    PD_payoff =[1,5;
                0,3]; %payoff matrix for PD game
    ErrorProb = 0.1; % probability of error in the move
    p_next_round=0.99654; % prob. of a subsequent round
    
%% User defined functions specifying strategies    
% funtions take the my and opponent's history of the moves 
% as an input and produce my move as an output 

strategy = {@AllC, @AllD, @TFT, @Grim};% list of strategies

    function move=AllC(My_hist, Opp_hist)
        move=Cooperate; % always cooperate (regardless of history)
    end;
    
    function move=AllD(My_hist, Opp_hist)
        move=Defect; % always defect (regardless of history)
    end;

    function move=TFT(My_hist, Opp_hist)
        if length(My_hist)==0 % if first move
            move=Cooperate; % cooperate on the first move
        else % not first move
            move=Opp_hist(end); % repeat opponent's last move
        end;
    end;
    
    function move=Grim(My_hist, Opp_hist)
        if any(Opp_hist==Defect) % if opponent ever defected
            move=Defect;
        else
            move=Cooperate;
        end;
    end;

%% Init and auxiliar variables
    Defect = 1;
    Cooperate = 2; 
    % with the above notation, PD_payoff(Defect, Cooperate)
    % determines the payoff to a player that defected if the other
    % cooperated
    % for errors, note that 3 - Movetype = OppositeMoveType
    Nplayers = length(strategy); % how many players entered
    score = zeros(1,Nplayers);   % init of scores as 0
    
%% actual tournament (round robin)
    for P1 = 1:Nplayers       % all players will play
        for P2 = P1:Nplayers  % with every other player
            %initialize the play between P1 and P2
            P1hist = []; P2hist = []; % no history so far
            P1score = 0; P2score = 0; % no score so far
            prob=0; % first round will be played
            % play the PD between P1 and P2
            while prob < p_next_round
			% get the moves
                P1move = strategy{P1}(P1hist, P2hist); 
                P2move = strategy{P2}(P2hist, P1hist); 
                % account for errors in the environment
                if rand(1) < ErrorProb % if error occured
                    P1move = 3-P1move; % P1 will do the opposite
                end
                if rand(1) < ErrorProb 
			    % if (another and independent) error 
                    P2move = 3-P2move; % P2 will do the opposite
                end
                % update the scores
                P1score = P1score + PD_payoff(P1move, P2move);
                P2score = P2score + PD_payoff(P2move, P1move);
                P1hist = [P1hist, P1move]; % update the history
                P2hist = [P2hist, P2move]; % update the history
                prob = rand(1); % if a subsequent round will be played
            end; % while prob < p_next_round
            % update players' scores by the average received
            score(P1) = score(P1) + P1score/length(P1hist);
            if P2>P1
                score(P2) = score(P2) + P2score/length(P2hist);
            end;
        end; % for P2
    end; % for P1
%% display the outcomes
for P=1:Nplayers
    disp(['Strategy ' num2str(P) ' got ' num2str(score(P)) ' points'])
end

end

